from typing import Iterable, List, Optional, Protocol

from lxml import etree as ET
from shapely.geometry.base import BaseGeometry

from imxInsights.domain.area.areaClassifiedTypeEnum import AreaClassifiedTypeEnum
from imxInsights.domain.area.imxProjectArea import ImxProjectArea
from imxInsights.domain.models.imxGeometry import ImxGeometry
from imxInsights.domain.models.imxMetadata import ImxObjectMetadata
from imxInsights.domain.models.ImxRailConnectionInfo import RailConnectionInfos
from imxInsights.domain.models.imxRelation import ImxReffedObjects
from imxInsights.repo.tree.imxTopoTreeObject import ImxTopoObject


class ImxObjectProtocol(Protocol):
    @property
    def properties(self) -> dict[str, str]:
        ...
        yield

    @property
    def metadata(self) -> ImxObjectMetadata:
        ...
        yield

    @property
    def geometry(self) -> Optional[ImxGeometry]:
        ...
        yield

    @property
    def rail_connection_infos(self) -> Optional[RailConnectionInfos]:
        ...
        yield

    @property
    def track_fragments(self) -> Optional[RailConnectionInfos]:
        ...
        yield

    @property
    def demarcation_markers(self) -> Optional[RailConnectionInfos]:
        ...
        yield

    @property
    def reffed_objects(self) -> ImxReffedObjects:
        ...
        yield

    @property
    def topo_object(self) -> Optional[ImxTopoObject]:
        ...
        yield

    @property
    def area(self) -> Optional[ImxProjectArea]:
        ...
        yield

    @property
    def area_classify_type(self) -> Optional[AreaClassifiedTypeEnum]:
        ...
        yield

    @property
    def parent(self) -> Optional["ImxObjectProtocol"]:
        ...
        yield

    @property
    def children(self) -> List["ImxObjectProtocol"]:
        ...
        yield

    @property
    def element(self) -> ET.Element:
        ...
        yield

    @property
    def tag(self) -> str:
        ...
        yield

    @property
    def path(self) -> str:
        ...
        yield

    @property
    def puic(self) -> str:
        ...
        yield

    @property
    def name(self) -> str:
        ...
        yield

    @property
    def shapely(self) -> BaseGeometry:
        ...
        yield

    def _parents_generator(self) -> Iterable["ImxObjectProtocol"]:
        ...
        yield

    def register_topo_object(self, topo_object: ImxTopoObject):
        ...
        yield

    def can_compare(self, other: Optional["ImxObjectProtocol"]) -> bool:
        ...
        yield

    @staticmethod
    def lookup_tree_from_root_element(start: ET.Element) -> list["ImxObjectProtocol"]:
        ...
        yield
